
boolean setConfig(char* param, char* value) {
  // sets the corresponding config variable for 'param' to new value
  // does not trigger saving to flash
  // does not distinguish between config and config2 as this is only split on flash and web-interface

  if (serialdebug) {
    Serial.print("setConfig - '");
    Serial.print(param);
    Serial.print("' to '");
    Serial.print(value);
    Serial.println("'");
  }

  // values
  if ( strcmp(param, "temp") == 0 ) {
    float valueFloat = round(atof(value) * 2.0) / 2.0;
    setTempTo(valueFloat);
  }
  else if ( strcmp(param, "tempLow") == 0 ) {
    float valueFloat = round(atof(value) * 2.0) / 2.0;
    setTempLowTo(valueFloat);
  }
  else if ( strcmp(param, "tempLow2") == 0 ) {
    float valueFloat = round(atof(value) * 2.0) / 2.0;
    setTempLow2To(valueFloat);
  }
  else if ( strcmp(param, "mode") == 0 ) {
    int val = atoi(value);
    if (val >= 0 && val <= 3) {
      setHeatingmodeTo(val);
    }
  }

  // conf DEV
  else if ( strcmp(param, "devName") == 0 ) {
    if (strlen(value) >= 4) strlcpy(deviceName, value, sizeof(deviceName));
  }
  else if ( strcmp(param, "hostName") == 0 ) {
    if (strlen(value) >= 4) strlcpy(hostName, value, sizeof(hostName));
  }
  else if ( strcmp(param, "wifiappw") == 0 ) {
    if (strlen(value) >= 8) strlcpy(wifiAPModePassword, value, sizeof(wifiAPModePassword));
  }

  // conf WEB
  else if ( strcmp(param, "apiToken") == 0 ) {
    strlcpy(http_token, value, sizeof(http_token));
  }
  else if ( strcmp(param, "httpUA") == 0 ) {
    strlcpy(http_user, value, sizeof(http_user));
  }
  else if ( strcmp(param, "httpPA") == 0 ) {
    strlcpy(http_pass, value, sizeof(http_pass));
  }
  else if ( strcmp(param, "httpAuth") == 0 ) {
    if (atoi(value) == 1) http_user_auth = true;
    else http_user_auth = false;
  }
  else if ( strcmp(param, "httpU1") == 0 ) {
    strlcpy(http_user1, value, sizeof(http_user1));
  }
  else if ( strcmp(param, "httpP1") == 0 ) {
    strlcpy(http_pass1, value, sizeof(http_pass1));
  }
  else if ( strcmp(param, "httpU2") == 0 ) {
    strlcpy(http_user2, value, sizeof(http_user2));
  }
  else if ( strcmp(param, "httpP2") == 0 ) {
    strlcpy(http_pass2, value, sizeof(http_pass2));
  }

  // conf MQTT
  else if ( strcmp(param, "mqttEnable") == 0 ) {
    if (atoi(value) == 1) mqtt_enable = true;
    else mqtt_enable = false;
  }
  else if ( strcmp(param, "mqttHost") == 0 ) {
    strlcpy(mqtt_server, value, sizeof(mqtt_server));
  }
  else if ( strcmp(param, "mqttPort") == 0 ) {
    mqtt_port = atoi(value);
  }
  else if ( strcmp(param, "mqttUser") == 0 ) {
    strlcpy(mqtt_user, value, sizeof(mqtt_user));
  }
  else if ( strcmp(param, "mqttPass") == 0 ) {
    strlcpy(mqtt_pass, value, sizeof(mqtt_pass));
  }
  else if ( strcmp(param, "inTop") == 0 ) {
    strlcpy(mqtt_topic_in, value, sizeof(mqtt_topic_in));
  }
  else if ( strcmp(param, "outTop") == 0 ) {
    strlcpy(mqtt_topic_out, value, sizeof(mqtt_topic_out));
  }
  else if ( strcmp(param, "outRet") == 0 ) {
    if (atoi(value) == 1) mqtt_outRetain = true;
    else mqtt_outRetain = false;
  }
  else if ( strcmp(param, "willTop") == 0 ) {
    strlcpy(mqtt_willTopic, value, sizeof(mqtt_willTopic));
  }
  else if ( strcmp(param, "willQos") == 0 ) {
    int tmpval = atoi(value);
    if (tmpval >= 0 && tmpval <= 2) mqtt_willQos = tmpval;
  }
  else if ( strcmp(param, "willRet") == 0 ) {
    if (atoi(value) == 1) mqtt_willRetain = true;
    else mqtt_willRetain = false;
  }
  else if ( strcmp(param, "willMsg") == 0 ) {
    strlcpy(mqtt_willMsg, value, sizeof(mqtt_willMsg));
  }
  else if ( strcmp(param, "connMsg") == 0 ) {
    strlcpy(mqtt_connMsg, value, sizeof(mqtt_connMsg));
  }
  else if ( strcmp(param, "hbEnable") == 0 ) {
    if (atoi(value) == 1) mqtt_enable_heartbeat = true;
    else mqtt_enable_heartbeat = false;
  }
  else if ( strcmp(param, "hbReconn") == 0 ) {
    int tmpval = atoi(value);
    if (tmpval == 0) mqtt_heartbeat_maxage_reconnect = 0;
    else if (tmpval >= 2 && tmpval <= 10) mqtt_heartbeat_maxage_reconnect = tmpval * 60000;
  }
  else if ( strcmp(param, "hbReboot") == 0 ) {
    int tmpval = atoi(value);
    if (tmpval == 0) mqtt_heartbeat_maxage_reboot = 0;
    else if (tmpval >= 10 && tmpval <= 120) mqtt_heartbeat_maxage_reboot = tmpval * 60000;
  }

  // conf BAS
  else if ( strcmp(param, "autoSaveTemp") == 0 ) {
    if (atoi(value) == 1) autoSaveSetTemp = true;
    else autoSaveSetTemp = false;
  }
  else if ( strcmp(param, "autoSaveMode") == 0 ) {
    if (atoi(value) == 1) autoSaveHeatingMode = true;
    else autoSaveHeatingMode = false;
  }
  else if ( strcmp(param, "tempMin") == 0 ) {
    float valueFloat = round(atof(value) * 2.0) / 2.0;
    if (valueFloat >= 10 && valueFloat <= 16) {
      setTempMin = valueFloat;
    }
  }
  else if ( strcmp(param, "tempMax") == 0 ) {
    float valueFloat = round(atof(value) * 2.0) / 2.0;
    if (valueFloat >= 18 && valueFloat <= 30) {
      setTempMax = valueFloat;
    }
  }
  else if ( strcmp(param, "measInt") == 0 ) {
    int valueInt = atoi(value);
    if (valueInt >= 5 && valueInt <= 120) {
      measureInterval = valueInt;
    }
  }
  else if ( strcmp(param, "dispInt") == 0 ) {
    int valueInt = atoi(value);
    if (valueInt >= 2 && valueInt <= 120) {
      displayInterval = valueInt;
    }
  }
  else if ( strcmp(param, "dispTout") == 0 ) {
    int valueInt = atoi(value);
    if (valueInt >= 2 && valueInt <= 1200) {
      displayTimeout = valueInt;
    }
  }
  else if ( strcmp(param, "PIRenDisp") == 0 ) {
    int valueInt = atoi(value);
    if (valueInt == 1) PIR_enablesDisplay = true;
    else PIR_enablesDisplay = false;
  }
  else if ( strcmp(param, "togTHdisp") == 0 ) {
    int valueInt = atoi(value);
    if (valueInt == 1) togglingTempHumAIDisplay = true;
    else togglingTempHumAIDisplay = false;
  }

  // conf ADV
  else if ( strcmp(param, "minOffTime") == 0 ) {
    heatingMinOffTime = atoi(value);
  }
  else if ( strcmp(param, "tempDec") == 0 ) {
    float valueFloat = atof(value);
    if (valueFloat >= 0.0 && valueFloat <= 1.5) {
      setTempDecreaseVal = valueFloat;
    }
  }
  else if ( strcmp(param, "hyst") == 0 ) {
    float valueFloat = atof(value);
    if (valueFloat >= 0.1 && valueFloat <= 4.0) {
      hysteresis = valueFloat;
    }
  }
  else if ( strcmp(param, "tempCorr") == 0 ) {
    float valueFloat = atof(value);
    if (valueFloat >= -5.0 && valueFloat <= 5.0) {
      tempCorrVal = valueFloat;
    }
  }
  else if ( strcmp(param, "humCorr") == 0 ) {
    int valueInt = atoi(value);
    if (valueInt >= -40 && valueInt <= 40) {
      humCorrVal = valueInt;
    }
  }
  else if ( strcmp(param, "offMsg") == 0 ) {
    strlcpy(offMessage, value, sizeof(offMessage));
  }
  else if ( strcmp(param, "modename0") == 0 ) {
    strlcpy(modename0, value, sizeof(modename0));
  }
  else if ( strcmp(param, "modename1") == 0 ) {
    strlcpy(modename1, value, sizeof(modename1));
  }
  else if ( strcmp(param, "psetname0") == 0 ) {
    strlcpy(psetname0, value, sizeof(psetname0));
  }
  else if ( strcmp(param, "psetname1") == 0 ) {
    strlcpy(psetname1, value, sizeof(psetname1));
  }
  else if ( strcmp(param, "psetname2") == 0 ) {
    strlcpy(psetname2, value, sizeof(psetname2));
  }
  else if ( strcmp(param, "itemplab") == 0 ) {
    strlcpy(itemplab, value, sizeof(itemplab));
  }
  else if ( strcmp(param, "otemplab") == 0 ) {
    strlcpy(otemplab, value, sizeof(otemplab));
  }
  

  // conf ADD
  else if ( strcmp(param, "outTempTop") == 0 ) {
    strlcpy(outTemp_topic_in, value, sizeof(outTemp_topic_in));
  }
  else if ( strcmp(param, "outHumTop") == 0 ) {
    strlcpy(outHum_topic_in, value, sizeof(outHum_topic_in));
  }
  else if ( strcmp(param, "PIRTop") == 0 ) {
    strlcpy(mqtt_topic_pir, value, sizeof(mqtt_topic_pir));
  }
  else if ( strcmp(param, "PIROnPld") == 0 ) {
    strlcpy(mqtt_payload_pir_on, value, sizeof(mqtt_payload_pir_on));
  }
  else if ( strcmp(param, "PIROffPld") == 0 ) {
    strlcpy(mqtt_payload_pir_off, value, sizeof(mqtt_payload_pir_off));
  }
}

void getConfig(char* param) {
  // gets and prints the corresponding config variable for 'param'

  if (serialdebug) {
    Serial.print("getConfig - '");
    Serial.print(param);
    Serial.println("'");
  }

  char buf[101];

  // values
  if ( strcmp(param, "temp") == 0 ) {
    char buf2[11];
    dtostrf(setTemp, 2, 1, buf2);
    sprintf(buf, "setTemp: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "tempLow") == 0 ) {
    char buf2[11];
    dtostrf(setTempLow, 2, 1, buf2);
    sprintf(buf, "setTempLow: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "tempLow2") == 0 ) {
    char buf2[11];
    dtostrf(setTempLow2, 2, 1, buf2);
    sprintf(buf, "setTempLow2: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "mode") == 0 ) {
    sprintf(buf, "heatingMode: '%d'", heatingMode);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "preset") == 0 ) {
    sprintf(buf, "preset: '%d'", preset);
    mqttSendLog(buf);
  }

  //confdata
  else if ( strcmp(param, "devName") == 0 ) {
    sprintf(buf, "devName: '%s'", deviceName);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "hostName") == 0 ) {
    sprintf(buf, "hostName: '%s'", hostName);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "wifiappw") == 0 ) {
    sprintf(buf, "wifiappw: '%s'", wifiAPModePassword);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "httpUser") == 0 ) {
    sprintf(buf, "httpUser: '%s'", http_user);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "httpPass") == 0 ) {
    sprintf(buf, "httpPass: '%s'", http_pass);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "httpToken") == 0 ) {
    sprintf(buf, "httpToken: '%s'", http_token);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "mqttHost") == 0 ) {
    sprintf(buf, "mqttHost: '%s'", mqtt_server);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "mqttPort") == 0 ) {
    sprintf(buf, "mqttPort: '%s'", mqtt_port);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "mqttUser") == 0 ) {
    sprintf(buf, "mqttUser: '%s'", mqtt_user);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "mqttPass") == 0 ) {
    sprintf(buf, "mqttPass: '%s'", mqtt_pass);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "inTop") == 0 ) {
    sprintf(buf, "inTop: '%s'", mqtt_topic_in);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "outTop") == 0 ) {
    sprintf(buf, "outTop: '%s'", mqtt_topic_out);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "outRet") == 0 ) {
    char buf2[11];
    if (mqtt_outRetain) strcpy(buf2, "1");
    else strcpy(buf2, "0");
    sprintf(buf, "outRet: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "willTop") == 0 ) {
    sprintf(buf, "willTop: '%s'", mqtt_willTopic);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "willQos") == 0 ) {
    sprintf(buf, "willQos: '%d'", mqtt_willQos);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "willRet") == 0 ) {
    char buf2[11];
    if (mqtt_willRetain) strcpy(buf2, "1");
    else strcpy(buf2, "0");
    sprintf(buf, "willRet: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "willMsg") == 0 ) {
    sprintf(buf, "willMsg: '%s'", mqtt_willMsg);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "connMsg") == 0 ) {
    sprintf(buf, "connMsg: '%s'", mqtt_connMsg);
    mqttSendLog(buf);
  }

  //confdata2
  else if ( strcmp(param, "outTempTop") == 0 ) {
    sprintf(buf, "outTempTop: '%s'", outTemp_topic_in);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "outHumTop") == 0 ) {
    sprintf(buf, "outHumTop: '%s'", outHum_topic_in);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "PIRTop") == 0 ) {
    sprintf(buf, "PIRTop: '%s'", mqtt_topic_pir);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "autoSaveTemp") == 0 ) {
    char buf2[11];
    if (autoSaveSetTemp) strcpy(buf2, "1");
    else strcpy(buf2, "0");
    sprintf(buf, "autoSaveTemp: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "autoSaveMode") == 0 ) {
    char buf2[11];
    if (autoSaveHeatingMode) strcpy(buf2, "1");
    else strcpy(buf2, "0");
    sprintf(buf, "autoSaveMode: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "minOffTime") == 0 ) {
    sprintf(buf, "minOffTime: '%d'", heatingMinOffTime);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "tempMin") == 0 ) {
    char buf2[11];
    dtostrf(setTempMin, 2, 1, buf2);
    sprintf(buf, "tempMin: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "tempMax") == 0 ) {
    char buf2[11];
    dtostrf(setTempMax, 2, 1, buf2);
    sprintf(buf, "tempMax: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "tempDec") == 0 ) {
    char buf2[11];
    dtostrf(setTempDecreaseVal, 2, 1, buf2);
    sprintf(buf, "tempDec: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "hyst") == 0 ) {
    char buf2[11];
    dtostrf(hysteresis, 2, 1, buf2);
    sprintf(buf, "hyst: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "tempCorr") == 0 ) {
    char buf2[11];
    dtostrf(tempCorrVal, 2, 1, buf2);
    sprintf(buf, "tempCorr: '%s'", buf2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "humCorr") == 0 ) {
    sprintf(buf, "humCorr: '%d'", humCorrVal);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "measInt") == 0 ) {
    sprintf(buf, "measInt: '%d'", measureInterval);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "dispInt") == 0 ) {
    sprintf(buf, "dispInt: '%d'", displayInterval);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "dispTout") == 0 ) {
    sprintf(buf, "dispTout: '%d'", displayTimeout);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "offMsg") == 0 ) {
    sprintf(buf, "offMsg: '%s'", offMessage);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "modename0") == 0 ) {
    sprintf(buf, "modename0: '%s'", modename0);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "modename1") == 0 ) {
    sprintf(buf, "modename1: '%s'", modename1);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "psetname0") == 0 ) {
    sprintf(buf, "psetname0: '%s'", psetname0);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "psetname1") == 0 ) {
    sprintf(buf, "psetname1: '%s'", psetname1);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "psetname2") == 0 ) {
    sprintf(buf, "psetname2: '%s'", psetname2);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "itemplab") == 0 ) {
    sprintf(buf, "itemplab: '%s'", itemplab);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "otemplab") == 0 ) {
    sprintf(buf, "otemplab: '%s'", otemplab);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "PIRenDisp") == 0 ) {
    char buf2[11];
    if (PIR_enablesDisplay) strcpy(buf2, "1");
    else strcpy(buf2, "0");
    sprintf(buf, "pirEnDisp: '%d'", PIR_enablesDisplay);
    mqttSendLog(buf);
  }
  else if ( strcmp(param, "togTHdisp") == 0 ) {
    char buf2[11];
    if (togglingTempHumAIDisplay) strcpy(buf2, "1");
    else strcpy(buf2, "0");
    sprintf(buf, "togTHdisp: '%d'", togglingTempHumAIDisplay);
    mqttSendLog(buf);
  }


}

void printConfigDev() {
  // prints current config vars to serial
  Serial.println("\nconfDataDev:");
  Serial.print("devName: ");
  Serial.println(deviceName);
  Serial.print("hostName: ");
  Serial.println(hostName);
  Serial.print("wifiappw: ");
  Serial.println(wifiAPModePassword);
}

void printConfigWeb() {
  // prints current config vars to serial
  Serial.println("\nconfDataWeb:");
  Serial.print("apiToken: ");
  Serial.println(http_token);
  Serial.print("httpUA: ");
  Serial.println(http_user);
  Serial.print("httpPA: ");
  Serial.println(http_pass);
  Serial.print("httpAuth: ");
  Serial.println(http_user_auth);
  Serial.print("httpU1: ");
  Serial.println(http_user1);
  Serial.print("httpP1: ");
  Serial.println(http_pass1);
  Serial.print("httpU2: ");
  Serial.println(http_user2);
  Serial.print("httpP2: ");
  Serial.println(http_pass2);
}

void printConfigMqtt() {
  // prints current config vars to serial
  Serial.println("\nconfDataMqtt:");
  Serial.print("mqttEnable: ");
  Serial.println(mqtt_enable);
  Serial.print("mqttHost: ");
  Serial.println(mqtt_server);
  Serial.print("mqttPort: ");
  Serial.println(mqtt_port);
  Serial.print("mqttUser: ");
  Serial.println(mqtt_user);
  Serial.print("mqttPass: ");
  Serial.println(mqtt_pass);
  Serial.print("inTop: ");
  Serial.println(mqtt_topic_in);
  Serial.print("outTop: ");
  Serial.println(mqtt_topic_out);
  Serial.print("outRet: ");
  Serial.println(mqtt_outRetain);
  Serial.print("willTop: ");
  Serial.println(mqtt_willTopic);
  Serial.print("willQos: ");
  Serial.println(mqtt_willQos);
  Serial.print("willRet: ");
  Serial.println(mqtt_willRetain);
  Serial.print("willMsg: ");
  Serial.println(mqtt_willMsg);
  Serial.print("connMsg: ");
  Serial.println(mqtt_connMsg);
  Serial.print("hbEnable: ");
  Serial.println(mqtt_enable_heartbeat);
  Serial.print("hbReconn: ");
  Serial.println(mqtt_heartbeat_maxage_reconnect);
  Serial.print("hbReboot: ");
  Serial.println(mqtt_heartbeat_maxage_reboot);
}

boolean loadConfigDev() {
  char configFileName[] = "/confDev.json";
  if (SPIFFS.exists(configFileName)) {
    File configFile = SPIFFS.open(configFileName, "r");
    if (!configFile) {
      Serial.print(F("ERR: Failed to open file '"));
      Serial.print(configFileName);
      Serial.println("'");
      return false;
    }
    else {
      Serial.print("file '");
      Serial.print(configFileName);
      Serial.print("' opened. ");
      size_t size = configFile.size();
      Serial.print("size=");
      Serial.println(size);

#ifdef DEBUG_VERBOSE
      Serial.println(F("file content:"));
      while (configFile.available()) {
        String line = configFile.readStringUntil('\n');
        Serial.println(line);
      }
      Serial.println();
      configFile.close();
      File configFile = SPIFFS.open(configFileName, "r");
#endif

      if (size > 1000) {
        Serial.println(F("Config file size is too large"));
        return false;
      }
      
      std::unique_ptr<char[]> buf(new char[size]);
      configFile.readBytes(buf.get(), size);

      DynamicJsonDocument json(size + 50);
      DeserializationError error = deserializeJson(json, buf.get());
      if (error) {
        Serial.println(F("Failed to parse config file"));
        return false;
      }

      strlcpy(deviceName, json["devName"] | "", sizeof(deviceName));
      strlcpy(hostName, json["hostName"] | "", sizeof(hostName));
      strlcpy(wifiAPModePassword, json["wifiappw"] | "", sizeof(wifiAPModePassword));

      Serial.println(F("Loaded config values:"));
      printConfigDev();
      Serial.println();
      return true;
    }
    configFile.close();
  }
  else {
    Serial.print("file '");
    Serial.print(configFileName);
    Serial.println("' does not exist");
    return false;
  }
} // loadConfigDev

boolean loadConfigWeb() { // loadConfigWeb
  char configFileName[] = "/confWeb.json";
  if (SPIFFS.exists(configFileName)) {
    File configFile = SPIFFS.open(configFileName, "r");
    if (!configFile) {
      Serial.print(F("ERR: Failed to open file '"));
      Serial.print(configFileName);
      Serial.println("'");
      return false;
    }
    else {
      Serial.print("file '");
      Serial.print(configFileName);
      Serial.print("' opened. ");
      size_t size = configFile.size();
      Serial.print("size=");
      Serial.println(size);

#ifdef DEBUG_VERBOSE
      Serial.println(F("file content:"));
      while (configFile.available()) {
        String line = configFile.readStringUntil('\n');
        Serial.println(line);
      }
      Serial.println();
      configFile.close();
      File configFile = SPIFFS.open(configFileName, "r");
#endif

      if (size > 1000) {
        Serial.println(F("Config file size is too large"));
        return false;
      }

      std::unique_ptr<char[]> buf(new char[size]);
      configFile.readBytes(buf.get(), size);

      DynamicJsonDocument json(size + 50);
      DeserializationError error = deserializeJson(json, buf.get());
      if (error) {
        Serial.println(F("Failed to parse config file"));
        return false;
      }
      
      strlcpy(http_token, json["apiToken"] | "", sizeof(http_token));
      strlcpy(http_user, json["httpUA"] | "", sizeof(http_user));
      strlcpy(http_pass, json["httpPA"] | "", sizeof(http_pass));

      if ((json["httpAuth"] | 0) == 1) http_user_auth = true;
      else http_user_auth = false;

      strlcpy(http_user1, json["httpU1"] | "", sizeof(http_user1));
      strlcpy(http_pass1, json["httpP1"] | "", sizeof(http_pass1));
      strlcpy(http_user2, json["httpU2"] | "", sizeof(http_user2));
      strlcpy(http_pass2, json["httpP2"] | "", sizeof(http_pass2));

      Serial.println(F("Loaded config values:"));
      printConfigWeb();
      Serial.println();
      return true;
    }
    configFile.close();
  }
  else {
    Serial.print("file '");
    Serial.print(configFileName);
    Serial.println("' does not exist");
    return false;
  }
} // loadConfigWeb

boolean loadConfigMqtt() { // loadConfigMqtt
  char configFileName[] = "/confMqtt.json";
  if (SPIFFS.exists(configFileName)) {
    File configFile = SPIFFS.open(configFileName, "r");
    if (!configFile) {
      Serial.print(F("ERR: Failed to open file '"));
      Serial.print(configFileName);
      Serial.println("'");
      return false;
    }
    else {
      Serial.print("file '");
      Serial.print(configFileName);
      Serial.print("' opened. ");
      size_t size = configFile.size();
      Serial.print("size=");
      Serial.println(size);

#ifdef DEBUG_VERBOSE
      Serial.println(F("file content:"));
      while (configFile.available()) {
        String line = configFile.readStringUntil('\n');
        Serial.println(line);
      }
      Serial.println();
      configFile.close();
      File configFile = SPIFFS.open(configFileName, "r");
#endif

      if (size > 1000) {
        Serial.println(F("Config file size is too large"));
        return false;
      }
      
      std::unique_ptr<char[]> buf(new char[size]);
      configFile.readBytes(buf.get(), size);
      
      DynamicJsonDocument json(size + 50);
      DeserializationError error = deserializeJson(json, buf.get());
      if (error) {
        Serial.println(F("Failed to parse config file"));
        return false;
      }

      if ((json["mqttEnable"] | 0) == 1) mqtt_enable = true;
      else mqtt_enable = false;

      strlcpy(mqtt_server, json["mqttHost"] | "", sizeof(mqtt_server));
      mqtt_port = json["mqttPort"] | 1883;
      strlcpy(mqtt_user, json["mqttUser"] | "", sizeof(mqtt_user));
      strlcpy(mqtt_pass, json["mqttPass"] | "", sizeof(mqtt_pass));
      strlcpy(mqtt_topic_in, json["inTop"] | "", sizeof(mqtt_topic_in));
      strlcpy(mqtt_topic_out, json["outTop"] | "", sizeof(mqtt_topic_out));
      
      if ((json["outRet"] | 0) == 1) mqtt_outRetain = true;
      else mqtt_outRetain = false;
      
      strlcpy(mqtt_willTopic, json["willTop"] | "", sizeof(mqtt_willTopic));
      mqtt_willQos = json["willQos"] | 0;
      
      if ((json["willRet"] | 0) == 1) mqtt_willRetain = true;
      else mqtt_willRetain = false;
      
      strlcpy(mqtt_willMsg, json["willMsg"] | "", sizeof(mqtt_willMsg));
      strlcpy(mqtt_connMsg, json["connMsg"] | "", sizeof(mqtt_connMsg));
      
      if ((json["hbEnable"] | 0) == 1) mqtt_enable_heartbeat = true;
      else mqtt_enable_heartbeat = false;
      
      unsigned int tmpval;
      tmpval = json["hbReconn"] | 0;
      if (tmpval == 0) mqtt_heartbeat_maxage_reconnect = 0;
      else if (tmpval >= 2 && tmpval <= 10) mqtt_heartbeat_maxage_reconnect = tmpval * 60000;
      
      tmpval = json["hbReboot"] | 0;
      if (tmpval == 0) mqtt_heartbeat_maxage_reboot = 0;
      else if (tmpval >= 2 && tmpval <= 10) mqtt_heartbeat_maxage_reboot = tmpval * 60000;
      
      Serial.println(F("Loaded config values:"));
      printConfigMqtt();
      return true;
    }
    configFile.close();
  }
  else {
    Serial.print("file '");
    Serial.print(configFileName);
    Serial.println("' does not exist");
    return false;
  }
} // loadConfigMqtt

boolean loadConfigBas() {
  char configFileName[] = "/confBas.json";
  if (SPIFFS.exists(configFileName)) {
    File configFile = SPIFFS.open(configFileName, "r");
    if (!configFile) {
      Serial.print(F("ERR: Failed to open file '"));
      Serial.print(configFileName);
      Serial.println("'");
      return false;
    }
    else {
      Serial.print("file '");
      Serial.print(configFileName);
      Serial.print("' opened. ");
      size_t size = configFile.size();
      Serial.print("size=");
      Serial.println(size);

#ifdef DEBUG_VERBOSE
      Serial.println("file content:");
      while (configFile.available()) {
        String line = configFile.readStringUntil('\n');
        Serial.println(line);
      }
      Serial.println();
      configFile.close();
      File configFile = SPIFFS.open(configFileName, "r");
#endif
      
      if (size > 1000) {
        Serial.println(F("Config file size is too large"));
        return false;
      }

      std::unique_ptr<char[]> buf(new char[size]);
      configFile.readBytes(buf.get(), size);
      
      DynamicJsonDocument json(size + 50);
      DeserializationError error = deserializeJson(json, buf.get());
      if (error) {
        Serial.println(F("Failed to parse config file"));
        return false;
      }

      if((json["autoSaveTemp"] | 0) == 1) autoSaveSetTemp = true;
      else autoSaveSetTemp = false;
      
      if((json["autoSaveMode"] | 0) == 1) autoSaveHeatingMode = true;
      else autoSaveHeatingMode = false;
      
      if((json["PIRenDisp"] | 0) == 1) PIR_enablesDisplay = true;
      else PIR_enablesDisplay = false;
      
      if((json["togTHdisp"] | 0) == 1) togglingTempHumAIDisplay = true;
      togglingTempHumAIDisplay = false;

      setTempMin = json["tempMin"] | 16;
      setTempMax = json["tempMax"] | 26;
      measureInterval = json["measInt"] | 15;
      displayInterval = json["dispInt"] | 5;
      displayTimeout = json["dispTout"] | 30;

      //Serial.println(F("Loaded config values:"));
      //printConfigBas();
      return true;
    }
    configFile.close();
  }
  else {
    Serial.print("file '");
    Serial.print(configFileName);
    Serial.println("' does not exist");
    return false;
  }
} // loadConfigBas

boolean loadConfigAdv() {
  char configFileName[] = "/confAdv.json";
  if (SPIFFS.exists(configFileName)) {
    File configFile = SPIFFS.open(configFileName, "r");
    if (!configFile) {
      Serial.print(F("ERR: Failed to open file '"));
      Serial.print(configFileName);
      Serial.println("'");
      return false;
    }
    else {
      Serial.print("file '");
      Serial.print(configFileName);
      Serial.print("' opened. ");
      size_t size = configFile.size();
      Serial.print("size=");
      Serial.println(size);

#ifdef DEBUG_VERBOSE
      Serial.println("file content:");
      while (configFile.available()) {
        String line = configFile.readStringUntil('\n');
        Serial.println(line);
      }
      Serial.println();
      configFile.close();
      File configFile = SPIFFS.open(configFileName, "r");
#endif

      if (size > 1000) {
        Serial.println(F("Config file size is too large"));
        return false;
      }

      std::unique_ptr<char[]> buf(new char[size]);
      configFile.readBytes(buf.get(), size);
      
      DynamicJsonDocument json(size + 50);
      DeserializationError error = deserializeJson(json, buf.get());
      if (error) {
        Serial.println(F("Failed to parse config file"));
        return false;
      }

      hysteresis = json["hyst"] | 0.1;
      heatingMinOffTime = json["minOffTime"] | 120;
      tempCorrVal = json["tempCorr"] | 0;
      humCorrVal = json["humCorr"] | 0;
      setTempDecreaseVal = json["tempDec"] | 0;

      strlcpy(offMessage, json["offMsg"] | "", sizeof(offMessage));

      strlcpy(itemplab, json["itemplab"] | "", sizeof(itemplab));
      strlcpy(otemplab, json["otemplab"] | "", sizeof(otemplab));

      strlcpy(modename0, json["modename0"] | "", sizeof(modename0));
      strlcpy(modename1, json["modename1"] | "", sizeof(modename1));
      strlcpy(psetname0, json["psetname0"] | "", sizeof(psetname0));
      strlcpy(psetname1, json["psetname1"] | "", sizeof(psetname1));
      strlcpy(psetname2, json["psetname2"] | "", sizeof(psetname2));

      //Serial.println("Loaded config values:");
      //printConfigWeb();
      return true;
    }
    configFile.close();
  }
  else {
    Serial.print("file '");
    Serial.print(configFileName);
    Serial.println("' does not exist");
    return false;
  }
} // loadConfigAdv

boolean loadConfigAdd() {
  char configFileName[] = "/confAdd.json";
  if (SPIFFS.exists(configFileName)) {
    File configFile = SPIFFS.open(configFileName, "r");
    if (!configFile) {
      Serial.print(F("ERR: Failed to open file '"));
      Serial.print(configFileName);
      Serial.println("'");
      return false;
    }
    else {
      Serial.print("file '");
      Serial.print(configFileName);
      Serial.print("' opened. ");
      size_t size = configFile.size();
      Serial.print("size=");
      Serial.println(size);

#ifdef DEBUG_VERBOSE
      Serial.println("file content:");
      while (configFile.available()) {
        String line = configFile.readStringUntil('\n');
        Serial.println(line);
      }
      Serial.println();
      configFile.close();
      File configFile = SPIFFS.open(configFileName, "r");
#endif

      if (size > 1000) {
        Serial.println(F("Config file size is too large"));
        return false;
      }

      std::unique_ptr<char[]> buf(new char[size]);
      configFile.readBytes(buf.get(), size);
      
      DynamicJsonDocument json(size + 50);
      DeserializationError error = deserializeJson(json, buf.get());
      if (error) {
        Serial.println(F("Failed to parse config file"));
        return false;
      }

      strlcpy(mqtt_topic_pir, json["PIRTop"] | "", sizeof(mqtt_topic_pir));
      strlcpy(mqtt_payload_pir_on, json["PIROnPld"] | "", sizeof(mqtt_payload_pir_on));
      strlcpy(mqtt_payload_pir_off, json["PIROffPld"] | "", sizeof(mqtt_payload_pir_off));
      strlcpy(outTemp_topic_in, json["outTempTop"] | "", sizeof(outTemp_topic_in));
      strlcpy(outHum_topic_in, json["outHumTop"] | "", sizeof(outHum_topic_in));
      
      //Serial.println("Loaded config values:");
      //printConfigWeb();
      return true;
    }
    configFile.close();
  }
  else {
    Serial.print("file '");
    Serial.print(configFileName);
    Serial.println("' does not exist");
    return false;
  }
} // loadConfigAdd

boolean loadSetTemp() { // loadSetTemp
  File configFile = SPIFFS.open("/setTemp", "r");
  if (!configFile) {
    Serial.println(F("ERR: Failed to open file /setTemp"));
    return false;
  }
  String s = configFile.readStringUntil('\n');
  configFile.close();
  float tmpSetTemp = s.toFloat();
  if ( tmpSetTemp >= setTempMin && tmpSetTemp <= setTempMax ) {
    setTemp = tmpSetTemp;
    setTempSaved = tmpSetTemp;
    return true;
  }
  else return false;
} // loadSetTemp

boolean loadSetTempLow() { // loadSetTempLow
  File configFile = SPIFFS.open("/setTempLow", "r");
  if (!configFile) {
    Serial.println(F("ERR: Failed to open file /setTempLow"));
    return false;
  }
  String s = configFile.readStringUntil('\n');
  configFile.close();
  float tmpSetTempLow = s.toFloat();
  if ( tmpSetTempLow >= setTempLowMin && tmpSetTempLow <= setTempLowMax ) {
    setTempLow = tmpSetTempLow;
    setTempLowSaved = tmpSetTempLow;
    return true;
  }
  else return false;
} // loadSetTempLow

boolean loadSetTempLow2() { // loadSetTempLow2
  File configFile = SPIFFS.open("/setTempLow2", "r");
  if (!configFile) {
    Serial.println(F("ERR: Failed to open file /setTempLow2"));
    return false;
  }
  String s = configFile.readStringUntil('\n');
  configFile.close();
  float tmpSetTempLow2 = s.toFloat();
  if ( tmpSetTempLow2 >= setTempLowMin && tmpSetTempLow2 <= setTempLowMax ) {
    setTempLow2 = tmpSetTempLow2;
    setTempLow2Saved = tmpSetTempLow2;
    return true;
  }
  else return false;
} // loadSetTempLow2

boolean loadHeatingMode() { // loadHeatingMode
  File configFile = SPIFFS.open("/heatingMode", "r");
  if (!configFile) {
    Serial.println(F("ERR: Failed to open file /heatingMode"));
    return false;
  }
  String s = configFile.readStringUntil('\n');
  configFile.close();
  int tmpHeatingMode = s.toInt();
  if ( tmpHeatingMode >= 0 && tmpHeatingMode <= 1 ) {
    heatingMode = tmpHeatingMode;
    heatingModeSaved = tmpHeatingMode;
    return true;
  }
  else return false;
} // loadHeatingMode

boolean loadPreset() { // loadPreset
  File configFile = SPIFFS.open("/preset", "r");
  if (!configFile) {
    Serial.println(F("ERR: Failed to open file /preset"));
    return false;
  }
  String s = configFile.readStringUntil('\n');
  configFile.close();
  int tmpPreset = s.toInt();
  if ( tmpPreset >= 0 && tmpPreset <= 2 ) {
    preset = tmpPreset;
    presetSaved = tmpPreset;
    return true;
  }
  else return false;
} // loadPreset

boolean saveConfigWeb() { // safeConfig
  char configFileName[] = "/confWeb.json";
  DynamicJsonDocument json(1050);
  
  json["apiToken"] = http_token;
  json["httpUA"] = http_user;
  json["httpPA"] = http_pass;

  if (http_user_auth) json["httpAuth"] = 1;
  else json["httpAuth"] = 0;

  json["httpU1"] = http_user1;
  json["httpP1"] = http_pass1;
  json["httpU2"] = http_user2;
  json["httpP2"] = http_pass2;

  yield();

  File configFile = SPIFFS.open(configFileName, "w");
  if (!configFile) {
    Serial.print("Failed to open file '");
    Serial.print(configFileName);
    Serial.println("' for writing");
    return false;
  }
  serializeJson(json, configFile);
  return true;
} // saveConfigWeb

boolean saveConfigMqtt() { // safeConfig
  char configFileName[] = "/confMqtt.json";
  DynamicJsonDocument json(1050);

  if (mqtt_enable) json["mqttEnable"] = 1;
  else json["mqttEnable"] = 0;

  json["mqttHost"] = mqtt_server;
  json["mqttPort"] = mqtt_port;
  json["mqttUser"] = mqtt_user;
  json["mqttPass"] = mqtt_pass;
  json["inTop"] = mqtt_topic_in;
  json["outTop"] = mqtt_topic_out;

  if (mqtt_outRetain) json["outRet"] = 1;
  else json["outRet"] = 0;

  json["willTop"] = mqtt_willTopic;
  json["willQos"] = mqtt_willQos;

  if (mqtt_willRetain) json["willRet"] = 1;
  else json["willRet"] = 0;

  json["willMsg"] = mqtt_willMsg;
  json["connMsg"] = mqtt_connMsg;

  if (mqtt_enable_heartbeat) json["hbEnable"] = 1;
  else json["hbEnable"] = 0;

  json["hbReconn"] = mqtt_heartbeat_maxage_reconnect / 60000;
  json["hbReboot"] = mqtt_heartbeat_maxage_reboot / 60000;

  yield();

  File configFile = SPIFFS.open(configFileName, "w");
  if (!configFile) {
    Serial.print("Failed to open file '");
    Serial.print(configFileName);
    Serial.println("' for writing");
    return false;
  }
  serializeJson(json, configFile);
  return true;
} // saveConfigMqtt

boolean saveConfigDev() { // safeConfig
  char configFileName[] = "/confDev.json";
  DynamicJsonDocument json(1050);

  json["devName"] = deviceName;
  json["hostName"] = hostName;
  json["wifiappw"] = wifiAPModePassword;

  yield();

  File configFile = SPIFFS.open(configFileName, "w");
  if (!configFile) {
    Serial.print("Failed to open file '");
    Serial.print(configFileName);
    Serial.println("' for writing");
    return false;
  }
  serializeJson(json, configFile);
  return true;
} // saveConfigDev

boolean saveConfigBas() { // safeConfig
  char configFileName[] = "/confBas.json";
  DynamicJsonDocument json(1050);

  if (autoSaveSetTemp) json["autoSaveTemp"] = 1;
  else json["autoSaveTemp"] = 0;

  if (autoSaveHeatingMode) json["autoSaveMode"] = 1;
  else json["autoSaveMode"] = 0;

  json["tempMin"] = setTempMin;
  json["tempMax"] = setTempMax;
  json["measInt"] = measureInterval;
  json["dispInt"] = displayInterval;
  json["dispTout"] = displayTimeout;

  if (PIR_enablesDisplay) json["PIRenDisp"] = 1;
  else json["PIRenDisp"] = 0;

  if (togglingTempHumAIDisplay) json["togTHdisp"] = 1;
  else json["togTHdisp"] = 0;

  yield();

  File configFile = SPIFFS.open(configFileName, "w");
  if (!configFile) {
    Serial.print("Failed to open file '");
    Serial.print(configFileName);
    Serial.println("' for writing");
    return false;
  }
  serializeJson(json, configFile);
  return true;
} // saveConfigBas

boolean saveConfigAdv() { // safeConfig
  char configFileName[] = "/confAdv.json";
  DynamicJsonDocument json(1050);

  json["hyst"] = hysteresis;
  json["minOffTime"] = heatingMinOffTime;
  json["tempCorr"] = tempCorrVal;
  json["humCorr"] = humCorrVal;
  json["tempDec"] = setTempDecreaseVal;
  json["offMsg"] = offMessage;
  json["itemplab"] = itemplab;
  json["otemplab"] = otemplab;
  json["modename0"] = modename0;
  json["modename1"] = modename1;
  json["psetname0"] = psetname0;
  json["psetname1"] = psetname1;
  json["psetname2"] = psetname2;

  yield();

  File configFile = SPIFFS.open(configFileName, "w");
  if (!configFile) {
    Serial.print("Failed to open file '");
    Serial.print(configFileName);
    Serial.println("' for writing");
    return false;
  }
  serializeJson(json, configFile);
  return true;
} // saveConfigAdv

boolean saveConfigAdd() { // safeConfig
  char configFileName[] = "/confAdd.json";
  DynamicJsonDocument json(1050);

  json["PIRTop"] = mqtt_topic_pir;
  json["PIROnPld"] = mqtt_payload_pir_on;
  json["PIROffPld"] = mqtt_payload_pir_off;
  json["outTempTop"] = outTemp_topic_in;
  json["outHumTop"] = outHum_topic_in;

  yield();

  File configFile = SPIFFS.open(configFileName, "w");
  if (!configFile) {
    Serial.print("Failed to open file '");
    Serial.print(configFileName);
    Serial.println("' for writing");
    return false;
  }
  serializeJson(json, configFile);
  return true;
} // saveConfigAdd


boolean saveSetTemp() { // saveSetTemp
  File configFile = SPIFFS.open("/setTemp", "w");
  if (!configFile) {
    Serial.println("Failed to open setTemp file for writing");
    return false;
  }
  configFile.println(setTemp);
  configFile.close();
  setTempSaved = setTemp;
  return true;
} // saveSetTemp

boolean saveSetTempLow() { // saveSetTempLow
  File configFile = SPIFFS.open("/setTempLow", "w");
  if (!configFile) {
    Serial.println("Failed to open setTempLow file for writing");
    return false;
  }
  configFile.println(setTempLow);
  configFile.close();
  setTempLowSaved = setTempLow;
  return true;
} // saveSetTempLow

boolean saveSetTempLow2() { // saveSetTempLow
  File configFile = SPIFFS.open("/setTempLow2", "w");
  if (!configFile) {
    Serial.println("Failed to open setTempLow2 file for writing");
    return false;
  }
  configFile.println(setTempLow2);
  configFile.close();
  setTempLow2Saved = setTempLow2;
  return true;
} // saveSetTempLow

boolean saveHeatingMode() { // saveHeatingMode
  File configFile = SPIFFS.open("/heatingMode", "w");
  if (!configFile) {
    Serial.println("Failed to open heatingMode file for writing");
    return false;
  }
  configFile.println(heatingMode);
  configFile.close();
  heatingModeSaved = heatingMode;
  return true;
} // saveHeatingMode


boolean savePreset() { // savePreset
  File configFile = SPIFFS.open("/preset", "w");
  if (!configFile) {
    Serial.println("Failed to open preset file for writing");
    return false;
  }
  configFile.println(preset);
  configFile.close();
  presetSaved = preset;
  return true;
} // savePreset

void deleteConfig() {
  Serial.println("deleting configuration");
  if (SPIFFS.remove("/formatted")) {
    Serial.println("config will be deleted after reboot");
    delay(100);
    ESP.restart();
  }
}

void loadConfigs() {
  loadConfigDev();
  loadConfigWeb();
  loadConfigMqtt();
  loadConfigBas();
  loadConfigAdv();
  loadConfigAdd();
}

void loadSavedValues() {
  loadSetTemp();
  loadSetTempLow();
  loadSetTempLow2();
  loadHeatingMode();
  loadPreset();
}
